/*
 * Copyright 2010 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */
/*
 * This file was generated.
 *  with google-apis-code-generator 1.3.0 (build: 2013-05-14 03:07:32 UTC)
 *  on 2013-05-16 at 16:31:14 UTC 
 */

package com.google.api.services.admin.reports;

import com.google.api.client.googleapis.GoogleUtils;
import com.google.api.client.googleapis.services.AbstractGoogleClientRequest;
import com.google.api.client.googleapis.services.GoogleClientRequestInitializer;
import com.google.api.client.googleapis.services.json.AbstractGoogleJsonClient;
import com.google.api.client.http.HttpRequestInitializer;
import com.google.api.client.http.HttpTransport;
import com.google.api.client.json.JsonFactory;
import com.google.api.client.json.JsonObjectParser;
import com.google.common.base.Preconditions;

/**
 * Service definition for Reports (reports_v1).
 *
 * <p>
 * Allows the administrators of Google Apps customers to fetch reports about the usage, collaboration, security and risk for their users.
 * </p>
 *
 * <p>
 * For more information about this service, see the
 * <a href="https://developers.google.com/google-apps/reports/" target="_blank">API Documentation</a>
 * </p>
 *
 * <p>
 * This service uses {@link ReportsRequestInitializer} to initialize global parameters via its
 * {@link Builder}.
 * </p>
 *
 * <p>
 * Upgrade warning: this class now extends {@link AbstractGoogleJsonClient}, whereas in prior
 * version 1.8 it extended {@link com.google.api.client.googleapis.services.GoogleClient}.
 * </p>
 *
 * @since 1.3
 * @author Google, Inc.
 */
@SuppressWarnings("javadoc")
public class Reports extends AbstractGoogleJsonClient {

  // Note: Leave this static initializer at the top of the file.
  static {
    Preconditions.checkState(GoogleUtils.VERSION.equals("1.13.2-beta"),
        "You are currently running with version %s of google-api-client. " +
        "You need version 1.13.2-beta of google-api-client to run version " +
        "1.13.2-beta of the Google Apps Reports API library.", GoogleUtils.VERSION);
  }

  /**
   * The default encoded root URL of the service. This is determined when the library is generated
   * and normally should not be changed.
   *
   * @since 1.7
   */
  public static final String DEFAULT_ROOT_URL = "https://www.googleapis.com/";

  /**
   * The default encoded service path of the service. This is determined when the library is
   * generated and normally should not be changed.
   *
   * @since 1.7
   */
  public static final String DEFAULT_SERVICE_PATH = "admin/reports/v1/";

  /**
   * The default encoded base URL of the service. This is determined when the library is generated
   * and normally should not be changed.
   * @deprecated (scheduled to be removed in 1.13)
   */
  @Deprecated
  public static final String DEFAULT_BASE_URL = DEFAULT_ROOT_URL + DEFAULT_SERVICE_PATH;

  /**
   * Constructor.
   *
   * <p>
   * Use {@link Builder} if you need to specify any of the optional parameters.
   * </p>
   *
   * @param transport HTTP transport
   * @param jsonFactory JSON factory
   * @param httpRequestInitializer HTTP request initializer or {@code null} for none
   * @since 1.7
   */
  public Reports(HttpTransport transport, JsonFactory jsonFactory,
      HttpRequestInitializer httpRequestInitializer) {
    super(transport,
        jsonFactory,
        DEFAULT_ROOT_URL,
        DEFAULT_SERVICE_PATH,
        httpRequestInitializer,
        false);
  }

  /**
   * @param transport HTTP transport
   * @param httpRequestInitializer HTTP request initializer or {@code null} for none
   * @param rootUrl root URL of the service
   * @param servicePath service path
   * @param jsonObjectParser JSON object parser
   * @param googleClientRequestInitializer Google request initializer or {@code null} for none
   * @param applicationName application name to be sent in the User-Agent header of requests or
   *        {@code null} for none
   * @param suppressPatternChecks whether discovery pattern checks should be suppressed on required
   *        parameters
   */
  Reports(HttpTransport transport,
      HttpRequestInitializer httpRequestInitializer,
      String rootUrl,
      String servicePath,
      JsonObjectParser jsonObjectParser,
      GoogleClientRequestInitializer googleClientRequestInitializer,
      String applicationName,
      boolean suppressPatternChecks) {
    super(transport,
        httpRequestInitializer,
        rootUrl,
        servicePath,
        jsonObjectParser,
        googleClientRequestInitializer,
        applicationName,
        suppressPatternChecks);
  }

  @Override
  protected void initialize(AbstractGoogleClientRequest<?> httpClientRequest) throws java.io.IOException {
    super.initialize(httpClientRequest);
  }

  /**
   * An accessor for creating requests from the Activities collection.
   *
   * <p>The typical use is:</p>
   * <pre>
   *   {@code Reports admin = new Reports(...);}
   *   {@code Reports.Activities.List request = admin.activities().list(parameters ...)}
   * </pre>
   *
   * @return the resource collection
   */
  public Activities activities() {
    return new Activities();
  }

  /**
   * The "activities" collection of methods.
   */
  public class Activities {

    /**
     * Retrieves a list of activities for a specific customer and application.
     *
     * Create a request for the method "activities.list".
     *
     * This request holds the parameters needed by the the admin server.  After setting any optional
     * parameters, call the {@link List#execute()} method to invoke the remote operation.
     *
     * @param userKey Represents the profile id or the user email for which the data should be filtered. When 'all' is
     *        specified as the userKey, it returns usageReports for all users.
     * @param applicationName Application name for which the events are to be retrieved.
     * @return the request
     */
    public List list(String userKey, String applicationName) throws java.io.IOException {
      List result = new List(userKey, applicationName);
      initialize(result);
      return result;
    }

    public class List extends ReportsRequest<com.google.api.services.admin.reports.model.Activities> {

      private static final String REST_PATH = "activity/users/{userKey}/applications/{applicationName}";

      private final java.util.regex.Pattern APPLICATIONNAME_PATTERN =
          java.util.regex.Pattern.compile("(admin)|(docs)");

      private final java.util.regex.Pattern STARTTIME_PATTERN =
          java.util.regex.Pattern.compile("(\\d\\d\\d\\d)-(\\d\\d)-(\\d\\d)T(\\d\\d):(\\d\\d):(\\d\\d)(?:\\.(\\d+))?(?:(Z)|([-+])(\\d\\d):(\\d\\d))");

      private final java.util.regex.Pattern FILTERS_PATTERN =
          java.util.regex.Pattern.compile("(.+[<,<=,==,>=,>,<>].+,)*(.+[<,<=,==,>=,>,<>].+)");

      private final java.util.regex.Pattern ENDTIME_PATTERN =
          java.util.regex.Pattern.compile("(\\d\\d\\d\\d)-(\\d\\d)-(\\d\\d)T(\\d\\d):(\\d\\d):(\\d\\d)(?:\\.(\\d+))?(?:(Z)|([-+])(\\d\\d):(\\d\\d))");

      /**
       * Retrieves a list of activities for a specific customer and application.
       *
       * Create a request for the method "activities.list".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link List#execute()} method to invoke the remote operation. <p> {@link
       * List#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
       * immediately after invoking the constructor. </p>
       *
       * @param userKey Represents the profile id or the user email for which the data should be filtered. When 'all' is
     *        specified as the userKey, it returns usageReports for all users.
       * @param applicationName Application name for which the events are to be retrieved.
       * @since 1.13
       */
      protected List(String userKey, String applicationName) {
        super(Reports.this, "GET", REST_PATH, null, com.google.api.services.admin.reports.model.Activities.class);
        this.userKey = Preconditions.checkNotNull(userKey, "Required parameter userKey must be specified.");
        this.applicationName = Preconditions.checkNotNull(applicationName, "Required parameter applicationName must be specified.");
        if (!getSuppressPatternChecks()) {
          Preconditions.checkArgument(APPLICATIONNAME_PATTERN.matcher(applicationName).matches(),
              "Parameter applicationName must conform to the pattern " +
              "(admin)|(docs)");
        }
      }

      @Override
      public com.google.api.client.http.HttpResponse executeUsingHead() throws java.io.IOException {
        return super.executeUsingHead();
      }

      @Override
      public com.google.api.client.http.HttpRequest buildHttpRequestUsingHead() throws java.io.IOException {
        return super.buildHttpRequestUsingHead();
      }

      @Override
      public List setAlt(String alt) {
        return (List) super.setAlt(alt);
      }

      @Override
      public List setFields(String fields) {
        return (List) super.setFields(fields);
      }

      @Override
      public List setKey(String key) {
        return (List) super.setKey(key);
      }

      @Override
      public List setOauthToken(String oauthToken) {
        return (List) super.setOauthToken(oauthToken);
      }

      @Override
      public List setPrettyPrint(Boolean prettyPrint) {
        return (List) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public List setQuotaUser(String quotaUser) {
        return (List) super.setQuotaUser(quotaUser);
      }

      @Override
      public List setUserIp(String userIp) {
        return (List) super.setUserIp(userIp);
      }

      /**
     * Represents the profile id or the user email for which the data should be filtered. When 'all'
     * is specified as the userKey, it returns usageReports for all users.
     */
      @com.google.api.client.util.Key
      private String userKey;

      /** Represents the profile id or the user email for which the data should be filtered. When 'all' is
     specified as the userKey, it returns usageReports for all users.
       */
      public String getUserKey() {
        return userKey;
      }

      /**
     * Represents the profile id or the user email for which the data should be filtered. When 'all'
     * is specified as the userKey, it returns usageReports for all users.
     */
      public List setUserKey(String userKey) {
        this.userKey = userKey;
        return this;
      }

      /** Application name for which the events are to be retrieved. */
      @com.google.api.client.util.Key
      private String applicationName;

      /** Application name for which the events are to be retrieved.
       */
      public String getApplicationName() {
        return applicationName;
      }

      /** Application name for which the events are to be retrieved. */
      public List setApplicationName(String applicationName) {
        if (!getSuppressPatternChecks()) {
          Preconditions.checkArgument(APPLICATIONNAME_PATTERN.matcher(applicationName).matches(),
              "Parameter applicationName must conform to the pattern " +
              "(admin)|(docs)");
        }
        this.applicationName = applicationName;
        return this;
      }

      /** Return events which occured at or after this time. */
      @com.google.api.client.util.Key
      private String startTime;

      /** Return events which occured at or after this time.
       */
      public String getStartTime() {
        return startTime;
      }

      /** Return events which occured at or after this time. */
      public List setStartTime(String startTime) {
        if (!getSuppressPatternChecks()) {
          Preconditions.checkArgument(STARTTIME_PATTERN.matcher(startTime).matches(),
              "Parameter startTime must conform to the pattern " +
              "(\\d\\d\\d\\d)-(\\d\\d)-(\\d\\d)T(\\d\\d):(\\d\\d):(\\d\\d)(?:\\.(\\d+))?(?:(Z)|([-+])(\\d\\d):(\\d\\d))");
        }
        this.startTime = startTime;
        return this;
      }

      /** IP Address of host where the event was performed. Supports both IPv4 and IPv6 addresses. */
      @com.google.api.client.util.Key
      private String actorIpAddress;

      /** IP Address of host where the event was performed. Supports both IPv4 and IPv6 addresses.
       */
      public String getActorIpAddress() {
        return actorIpAddress;
      }

      /** IP Address of host where the event was performed. Supports both IPv4 and IPv6 addresses. */
      public List setActorIpAddress(String actorIpAddress) {
        this.actorIpAddress = actorIpAddress;
        return this;
      }

      /** Number of activity records to be shown in each page. */
      @com.google.api.client.util.Key
      private Integer maxResults;

      /** Number of activity records to be shown in each page.

     [minimum: 1] [maximum: 1000]
       */
      public Integer getMaxResults() {
        return maxResults;
      }

      /** Number of activity records to be shown in each page. */
      public List setMaxResults(Integer maxResults) {
        this.maxResults = maxResults;
        return this;
      }

      /** Name of the event being queried. */
      @com.google.api.client.util.Key
      private String eventName;

      /** Name of the event being queried.
       */
      public String getEventName() {
        return eventName;
      }

      /** Name of the event being queried. */
      public List setEventName(String eventName) {
        this.eventName = eventName;
        return this;
      }

      /** Token to specify next page. */
      @com.google.api.client.util.Key
      private String pageToken;

      /** Token to specify next page.
       */
      public String getPageToken() {
        return pageToken;
      }

      /** Token to specify next page. */
      public List setPageToken(String pageToken) {
        this.pageToken = pageToken;
        return this;
      }

      /**
     * Event parameters in the form [parameter1 name][operator][parameter1 value],[parameter2
     * name][operator][parameter2 value],...
     */
      @com.google.api.client.util.Key
      private String filters;

      /** Event parameters in the form [parameter1 name][operator][parameter1 value],[parameter2
     name][operator][parameter2 value],...
       */
      public String getFilters() {
        return filters;
      }

      /**
     * Event parameters in the form [parameter1 name][operator][parameter1 value],[parameter2
     * name][operator][parameter2 value],...
     */
      public List setFilters(String filters) {
        if (!getSuppressPatternChecks()) {
          Preconditions.checkArgument(FILTERS_PATTERN.matcher(filters).matches(),
              "Parameter filters must conform to the pattern " +
              "(.+[<,<=,==,>=,>,<>].+,)*(.+[<,<=,==,>=,>,<>].+)");
        }
        this.filters = filters;
        return this;
      }

      /** Return events which occured at or before this time. */
      @com.google.api.client.util.Key
      private String endTime;

      /** Return events which occured at or before this time.
       */
      public String getEndTime() {
        return endTime;
      }

      /** Return events which occured at or before this time. */
      public List setEndTime(String endTime) {
        if (!getSuppressPatternChecks()) {
          Preconditions.checkArgument(ENDTIME_PATTERN.matcher(endTime).matches(),
              "Parameter endTime must conform to the pattern " +
              "(\\d\\d\\d\\d)-(\\d\\d)-(\\d\\d)T(\\d\\d):(\\d\\d):(\\d\\d)(?:\\.(\\d+))?(?:(Z)|([-+])(\\d\\d):(\\d\\d))");
        }
        this.endTime = endTime;
        return this;
      }

    }

  }

  /**
   * An accessor for creating requests from the CustomerUsageReports collection.
   *
   * <p>The typical use is:</p>
   * <pre>
   *   {@code Reports admin = new Reports(...);}
   *   {@code Reports.CustomerUsageReports.List request = admin.customerUsageReports().list(parameters ...)}
   * </pre>
   *
   * @return the resource collection
   */
  public CustomerUsageReports customerUsageReports() {
    return new CustomerUsageReports();
  }

  /**
   * The "customerUsageReports" collection of methods.
   */
  public class CustomerUsageReports {

    /**
     * Retrieves a report which is a collection of properties / statistics for a specific customer.
     *
     * Create a request for the method "customerUsageReports.get".
     *
     * This request holds the parameters needed by the the admin server.  After setting any optional
     * parameters, call the {@link Get#execute()} method to invoke the remote operation.
     *
     * @param date Represents the date in yyyy-mm-dd format for which the data is to be fetched.
     * @return the request
     */
    public Get get(String date) throws java.io.IOException {
      Get result = new Get(date);
      initialize(result);
      return result;
    }

    public class Get extends ReportsRequest<com.google.api.services.admin.reports.model.UsageReports> {

      private static final String REST_PATH = "usage/dates/{date}";

      private final java.util.regex.Pattern DATE_PATTERN =
          java.util.regex.Pattern.compile("(\\d){4}-(\\d){2}-(\\d){2}");

      private final java.util.regex.Pattern PARAMETERS_PATTERN =
          java.util.regex.Pattern.compile("(((accounts)|(gmail)|(calendar)|(docs)):.+,)*(((accounts)|(gmail)|(calendar)|(docs)):.+)");

      /**
       * Retrieves a report which is a collection of properties / statistics for a specific customer.
       *
       * Create a request for the method "customerUsageReports.get".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link Get#execute()} method to invoke the remote operation. <p> {@link
       * Get#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
       * immediately after invoking the constructor. </p>
       *
       * @param date Represents the date in yyyy-mm-dd format for which the data is to be fetched.
       * @since 1.13
       */
      protected Get(String date) {
        super(Reports.this, "GET", REST_PATH, null, com.google.api.services.admin.reports.model.UsageReports.class);
        this.date = Preconditions.checkNotNull(date, "Required parameter date must be specified.");
        if (!getSuppressPatternChecks()) {
          Preconditions.checkArgument(DATE_PATTERN.matcher(date).matches(),
              "Parameter date must conform to the pattern " +
              "(\\d){4}-(\\d){2}-(\\d){2}");
        }
      }

      @Override
      public com.google.api.client.http.HttpResponse executeUsingHead() throws java.io.IOException {
        return super.executeUsingHead();
      }

      @Override
      public com.google.api.client.http.HttpRequest buildHttpRequestUsingHead() throws java.io.IOException {
        return super.buildHttpRequestUsingHead();
      }

      @Override
      public Get setAlt(String alt) {
        return (Get) super.setAlt(alt);
      }

      @Override
      public Get setFields(String fields) {
        return (Get) super.setFields(fields);
      }

      @Override
      public Get setKey(String key) {
        return (Get) super.setKey(key);
      }

      @Override
      public Get setOauthToken(String oauthToken) {
        return (Get) super.setOauthToken(oauthToken);
      }

      @Override
      public Get setPrettyPrint(Boolean prettyPrint) {
        return (Get) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Get setQuotaUser(String quotaUser) {
        return (Get) super.setQuotaUser(quotaUser);
      }

      @Override
      public Get setUserIp(String userIp) {
        return (Get) super.setUserIp(userIp);
      }

      /** Represents the date in yyyy-mm-dd format for which the data is to be fetched. */
      @com.google.api.client.util.Key
      private String date;

      /** Represents the date in yyyy-mm-dd format for which the data is to be fetched.
       */
      public String getDate() {
        return date;
      }

      /** Represents the date in yyyy-mm-dd format for which the data is to be fetched. */
      public Get setDate(String date) {
        if (!getSuppressPatternChecks()) {
          Preconditions.checkArgument(DATE_PATTERN.matcher(date).matches(),
              "Parameter date must conform to the pattern " +
              "(\\d){4}-(\\d){2}-(\\d){2}");
        }
        this.date = date;
        return this;
      }

      /** Token to specify next page. */
      @com.google.api.client.util.Key
      private String pageToken;

      /** Token to specify next page.
       */
      public String getPageToken() {
        return pageToken;
      }

      /** Token to specify next page. */
      public Get setPageToken(String pageToken) {
        this.pageToken = pageToken;
        return this;
      }

      /**
     * Represents the application name, parameter name pairs to fetch in csv as
     * app_name1:param_name1, app_name2:param_name2.
     */
      @com.google.api.client.util.Key
      private String parameters;

      /** Represents the application name, parameter name pairs to fetch in csv as app_name1:param_name1,
     app_name2:param_name2.
       */
      public String getParameters() {
        return parameters;
      }

      /**
     * Represents the application name, parameter name pairs to fetch in csv as
     * app_name1:param_name1, app_name2:param_name2.
     */
      public Get setParameters(String parameters) {
        if (!getSuppressPatternChecks()) {
          Preconditions.checkArgument(PARAMETERS_PATTERN.matcher(parameters).matches(),
              "Parameter parameters must conform to the pattern " +
              "(((accounts)|(gmail)|(calendar)|(docs)):.+,)*(((accounts)|(gmail)|(calendar)|(docs)):.+)");
        }
        this.parameters = parameters;
        return this;
      }

    }

  }

  /**
   * An accessor for creating requests from the UserUsageReport collection.
   *
   * <p>The typical use is:</p>
   * <pre>
   *   {@code Reports admin = new Reports(...);}
   *   {@code Reports.UserUsageReport.List request = admin.userUsageReport().list(parameters ...)}
   * </pre>
   *
   * @return the resource collection
   */
  public UserUsageReport userUsageReport() {
    return new UserUsageReport();
  }

  /**
   * The "userUsageReport" collection of methods.
   */
  public class UserUsageReport {

    /**
     * Retrieves a report which is a collection of properties / statistics for a set of users.
     *
     * Create a request for the method "userUsageReport.get".
     *
     * This request holds the parameters needed by the the admin server.  After setting any optional
     * parameters, call the {@link Get#execute()} method to invoke the remote operation.
     *
     * @param userKey Represents the profile id or the user email for which the data should be filtered.
     * @param date Represents the date in yyyy-mm-dd format for which the data is to be fetched.
     * @return the request
     */
    public Get get(String userKey, String date) throws java.io.IOException {
      Get result = new Get(userKey, date);
      initialize(result);
      return result;
    }

    public class Get extends ReportsRequest<com.google.api.services.admin.reports.model.UsageReports> {

      private static final String REST_PATH = "usage/users/{userKey}/dates/{date}";

      private final java.util.regex.Pattern DATE_PATTERN =
          java.util.regex.Pattern.compile("(\\d){4}-(\\d){2}-(\\d){2}");

      private final java.util.regex.Pattern PARAMETERS_PATTERN =
          java.util.regex.Pattern.compile("(((accounts)|(gmail)|(calendar)|(docs)):.+,)*(((accounts)|(gmail)|(calendar)|(docs)):.+)");

      private final java.util.regex.Pattern FILTERS_PATTERN =
          java.util.regex.Pattern.compile("(((accounts)|(gmail)|(calendar)|(docs)):.+[<,<=,==,>=,>,!=].+,)*(((accounts)|(gmail)|(calendar)|(docs)):.+[<,<=,==,>=,>,!=].+)");

      /**
       * Retrieves a report which is a collection of properties / statistics for a set of users.
       *
       * Create a request for the method "userUsageReport.get".
       *
       * This request holds the parameters needed by the the admin server.  After setting any optional
       * parameters, call the {@link Get#execute()} method to invoke the remote operation. <p> {@link
       * Get#initialize(AbstractGoogleClientRequest)} must be called to initialize this instance
       * immediately after invoking the constructor. </p>
       *
       * @param userKey Represents the profile id or the user email for which the data should be filtered.
       * @param date Represents the date in yyyy-mm-dd format for which the data is to be fetched.
       * @since 1.13
       */
      protected Get(String userKey, String date) {
        super(Reports.this, "GET", REST_PATH, null, com.google.api.services.admin.reports.model.UsageReports.class);
        this.userKey = Preconditions.checkNotNull(userKey, "Required parameter userKey must be specified.");
        this.date = Preconditions.checkNotNull(date, "Required parameter date must be specified.");
        if (!getSuppressPatternChecks()) {
          Preconditions.checkArgument(DATE_PATTERN.matcher(date).matches(),
              "Parameter date must conform to the pattern " +
              "(\\d){4}-(\\d){2}-(\\d){2}");
        }
      }

      @Override
      public com.google.api.client.http.HttpResponse executeUsingHead() throws java.io.IOException {
        return super.executeUsingHead();
      }

      @Override
      public com.google.api.client.http.HttpRequest buildHttpRequestUsingHead() throws java.io.IOException {
        return super.buildHttpRequestUsingHead();
      }

      @Override
      public Get setAlt(String alt) {
        return (Get) super.setAlt(alt);
      }

      @Override
      public Get setFields(String fields) {
        return (Get) super.setFields(fields);
      }

      @Override
      public Get setKey(String key) {
        return (Get) super.setKey(key);
      }

      @Override
      public Get setOauthToken(String oauthToken) {
        return (Get) super.setOauthToken(oauthToken);
      }

      @Override
      public Get setPrettyPrint(Boolean prettyPrint) {
        return (Get) super.setPrettyPrint(prettyPrint);
      }

      @Override
      public Get setQuotaUser(String quotaUser) {
        return (Get) super.setQuotaUser(quotaUser);
      }

      @Override
      public Get setUserIp(String userIp) {
        return (Get) super.setUserIp(userIp);
      }

      /** Represents the profile id or the user email for which the data should be filtered. */
      @com.google.api.client.util.Key
      private String userKey;

      /** Represents the profile id or the user email for which the data should be filtered.
       */
      public String getUserKey() {
        return userKey;
      }

      /** Represents the profile id or the user email for which the data should be filtered. */
      public Get setUserKey(String userKey) {
        this.userKey = userKey;
        return this;
      }

      /** Represents the date in yyyy-mm-dd format for which the data is to be fetched. */
      @com.google.api.client.util.Key
      private String date;

      /** Represents the date in yyyy-mm-dd format for which the data is to be fetched.
       */
      public String getDate() {
        return date;
      }

      /** Represents the date in yyyy-mm-dd format for which the data is to be fetched. */
      public Get setDate(String date) {
        if (!getSuppressPatternChecks()) {
          Preconditions.checkArgument(DATE_PATTERN.matcher(date).matches(),
              "Parameter date must conform to the pattern " +
              "(\\d){4}-(\\d){2}-(\\d){2}");
        }
        this.date = date;
        return this;
      }

      /**
     * Represents the application name, parameter name pairs to fetch in csv as
     * app_name1:param_name1, app_name2:param_name2.
     */
      @com.google.api.client.util.Key
      private String parameters;

      /** Represents the application name, parameter name pairs to fetch in csv as app_name1:param_name1,
     app_name2:param_name2.
       */
      public String getParameters() {
        return parameters;
      }

      /**
     * Represents the application name, parameter name pairs to fetch in csv as
     * app_name1:param_name1, app_name2:param_name2.
     */
      public Get setParameters(String parameters) {
        if (!getSuppressPatternChecks()) {
          Preconditions.checkArgument(PARAMETERS_PATTERN.matcher(parameters).matches(),
              "Parameter parameters must conform to the pattern " +
              "(((accounts)|(gmail)|(calendar)|(docs)):.+,)*(((accounts)|(gmail)|(calendar)|(docs)):.+)");
        }
        this.parameters = parameters;
        return this;
      }

      /** Maximum number of results to return. Maximum allowed is 1000 */
      @com.google.api.client.util.Key
      private Long maxResults;

      /** Maximum number of results to return. Maximum allowed is 1000

     [maximum: 1000]
       */
      public Long getMaxResults() {
        return maxResults;
      }

      /** Maximum number of results to return. Maximum allowed is 1000 */
      public Get setMaxResults(Long maxResults) {
        this.maxResults = maxResults;
        return this;
      }

      /** Token to specify next page. */
      @com.google.api.client.util.Key
      private String pageToken;

      /** Token to specify next page.
       */
      public String getPageToken() {
        return pageToken;
      }

      /** Token to specify next page. */
      public Get setPageToken(String pageToken) {
        this.pageToken = pageToken;
        return this;
      }

      /** Represents the set of filters including parameter operator value. */
      @com.google.api.client.util.Key
      private String filters;

      /** Represents the set of filters including parameter operator value.
       */
      public String getFilters() {
        return filters;
      }

      /** Represents the set of filters including parameter operator value. */
      public Get setFilters(String filters) {
        if (!getSuppressPatternChecks()) {
          Preconditions.checkArgument(FILTERS_PATTERN.matcher(filters).matches(),
              "Parameter filters must conform to the pattern " +
              "(((accounts)|(gmail)|(calendar)|(docs)):.+[<,<=,==,>=,>,!=].+,)*(((accounts)|(gmail)|(calendar)|(docs)):.+[<,<=,==,>=,>,!=].+)");
        }
        this.filters = filters;
        return this;
      }

    }

  }

  /**
   * Builder for {@link Reports}.
   *
   * <p>
   * Implementation is not thread-safe.
   * </p>
   *
   * @since 1.3.0
   */
  public static final class Builder extends AbstractGoogleJsonClient.Builder {

    /**
     * Returns an instance of a new builder.
     *
     * @param transport HTTP transport
     * @param jsonFactory JSON factory
     * @param httpRequestInitializer HTTP request initializer or {@code null} for none
     * @since 1.7
     */
    public Builder(HttpTransport transport, JsonFactory jsonFactory,
        HttpRequestInitializer httpRequestInitializer) {
      super(
          transport,
          jsonFactory,
          DEFAULT_ROOT_URL,
          DEFAULT_SERVICE_PATH,
          httpRequestInitializer,
          false);
    }

    /** Builds a new instance of {@link Reports}. */
    @Override
    public Reports build() {
      return new Reports(getTransport(),
          getHttpRequestInitializer(),
          getRootUrl(),
          getServicePath(),
          getObjectParser(),
          getGoogleClientRequestInitializer(),
          getApplicationName(),
          getSuppressPatternChecks());
    }

    @Override
    public Builder setRootUrl(String rootUrl) {
      return (Builder) super.setRootUrl(rootUrl);
    }

    @Override
    public Builder setServicePath(String servicePath) {
      return (Builder) super.setServicePath(servicePath);
    }

    @Override
    public Builder setHttpRequestInitializer(HttpRequestInitializer httpRequestInitializer) {
      return (Builder) super.setHttpRequestInitializer(httpRequestInitializer);
    }

    @Override
    public Builder setApplicationName(String applicationName) {
      return (Builder) super.setApplicationName(applicationName);
    }

    @Override
    public Builder setSuppressPatternChecks(boolean suppressPatternChecks) {
      return (Builder) super.setSuppressPatternChecks(suppressPatternChecks);
    }

    /**
     * Set the {@link ReportsRequestInitializer}.
     *
     * @since 1.12
     */
    public Builder setReportsRequestInitializer(
        ReportsRequestInitializer reportsRequestInitializer) {
      return (Builder) super.setGoogleClientRequestInitializer(reportsRequestInitializer);
    }

    @Override
    public Builder setGoogleClientRequestInitializer(
        GoogleClientRequestInitializer googleClientRequestInitializer) {
      return (Builder) super.setGoogleClientRequestInitializer(googleClientRequestInitializer);
    }
  }
}
